print <<EOF;

Code Converter v.5
by Jonah Cohen  <http://jonah.ticalc.org>

EOF

if(scalar(@ARGV)<1)
{
	print <<EOF;
Syntax: $0 [-options] source.asm [dest.asm]
options:
   -t     indent using tabs
   -s[N]  indent using N spaces; N defaults to 1
   -a     align operands
   -d     don't align operands
   -u     UpperCase labels
   -l     lower_case labels
   -i     include #if 0, #endif blocks

EOF
	exit(0);
}

foreach (@ARGV)
{
	$file = $_;
	last unless /^-[tsaduli]/;
	
	$doindent = $& if /[ts]/;
	$doalign  = $& if /[ad]/;
	$docase	  = $& if /[ul]/;
	$include  = $& if /i/;
	
	$spaces   =  1 if /s/;
	$spaces   = $1 if /s(\d+)/;
}

&input;
&indent  if $doindent;
&align   if $doalign;
&parse   if $docase;
&replace if $docase;
&output;


sub input
{
	print "Reading in file...\n";
	
	open(INPUT, $file) || die "Can't read from $file\n";
	foreach(<INPUT>)
	{
		chomp;
		push @file,$_;
	}
	close(INPUT);
}

sub output
{
	print "Writing file...\n";
	
	open(OUTPUT, ">$ARGV[scalar(@ARGV)-1]") || die "Can't write to $ARGV[scalar(@ARGV)-1]\n";
	foreach(@file)
	{
		print OUTPUT "$_\n";
	}
	close(OUTPUT);
}

sub indent
{
	print "Indending...\n";
	
	$separator = "\t"          if $doindent =~ /t/;
	$separator = " " x $spaces if $doindent =~ /s/;
	
	$incomment = 0;
	for($i = 0; $i < scalar(@file); $i++)
	{
		unless($include)
		{
			$incomment = 1 if $file[$i] =~ /^#if 0/;
			$incomment = 0 if $file[$i] =~ /^#(else|elseif|endif)/;
		}
		
		$file[$i] =~ s/^\s+/$separator/ unless $incomments;
	}
}

sub align
{
	print "Aligning...\n"   if $doalign =~ /a/;
	print "Unaligning...\n" if $doalign =~ /d/;
	
	$separator = "\t" if $doalign =~ /a/;
	$separator = " "  if $doalign =~ /d/;
	
	$incomment = 0;
	for($i = 0; $i < scalar(@file); $i++)
	{
		unless($include)
		{
			$incomment = 1 if $file[$i] =~ /^#if 0/;
			$incomment = 0 if $file[$i] =~ /^#(else|elseif|endif)/;
		}
		
		$file[$i] =~ s/^(\s+\S+)\s+/$1$separator/ unless $incomments;
	}
}

sub parse
{
	print "Parsing labels...\n";
	
	$incomment = 0;
	for($i = 0; $i < scalar(@file); $i++)
	{
		unless($include)
		{
			$incomment = 1 if $file[$i] =~ /^#if 0/;
			$incomment = 0 if $file[$i] =~ /^#(else|elseif|endif)/;
		}
		
		&convert($&) if $incomments==0 && $file[$i] =~ /^\w+/;
	}
	
	chop $labels;
}

sub convert
{
	$_ = $_[0];
	$labels .= "$_|";
	$l = $_;
	
	if($docase =~ /u/)
	{
		@words = split /_/;
		$_ = "";
		foreach $word (@words) {$_ .= ucfirst $word;}
		s/^\d/_$&/;
	}
	else
	{
		s/([^A-Z])([A-Z]+)/$1_$2/g;
		tr/[A-Z]/[a-z]/;
	}
	
	$labels{$l} = $_;
}

sub replace
{
	print "Replacing labels...\n";
	
	$incomment = 0;
	for($i = 0; $i < scalar(@file); $i++)
	{
		unless($include)
		{
			$incomment = 1 if $file[$i] =~ /^#if 0/;
			$incomment = 0 if $file[$i] =~ /^#(else|elseif|endif)/;
		}
		
		$file[$i] =~ s/^([^;]*)\b($labels)\b/$1$labels{$2}/g unless $incomment;
		$file[$i] =~ s/^($labels)\b/$labels{$1}/             unless $incomment;
	}
}